package com.bazaarvoice.jolt.shiftr.reference;


/**
 * Reference is used by Shiftr when lookup up values from a WalkedPath (list of LiteralPathElements).
 *
 * Ex given a WalkedPath like :
 *  WalkedPath : [
 *    LiteralPathElement : [ "cdv-Tuna", "Tuna" ],   // This LiteralPathElement would be generated by a match of "cdv-*" and the key "cdv-Tuna"
 *    LiteralPahtElement : [ "Delicious" ]
 *  ]
 *
 *   &,  &0, &(0,0) would all evaluate to "Delicious"
 *   &1, &1, &(1,0) would all evaluate to "cdv-Tuna"
 *   &(1,1) would evaluate to "Tuna"
 *
 * The "canonical form" is "C(x,y)", where :
 *  C : the character used to determine the type of Reference
 *  x : pathIndex : which is how far up the walkedPath the look
 *  y : keyGroup : where 0 is the whole key, and 1 thru n smaller captured parts of the key
 *
 */
public interface Reference {

    public int getPathIndex();

    public int getKeyGroup();

    /**
     * Get the canonical form of this Reference.
     *
     * One of the uses of this method is to ensure that spec, does not contain "duplicate" keys, aka
     *  two keys that when you unroll the syntactic sugar, are the same thing.
     *
     * @return fully expanded String representation of this Reference
     */
    public String getCanonicalForm();
}
